package com.ym.mec.auth.web.controller;

import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.auth.config.constant.SecurityConstants;
import com.ym.mec.auth.service.SysRoleService;
import com.ym.mec.auth.service.SysUserRoleService;
import com.ym.mec.auth.service.SysUserService;
import com.ym.mec.auth.web.controller.queryInfo.SysUserQueryInfo;
import com.ym.mec.common.controller.BaseController;
import io.swagger.annotations.*;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.web.bind.annotation.*;

import java.util.Date;
import java.util.HashMap;

@RestController()
@RequestMapping("user")
@Api(tags = "会员服务")
public class UserController extends BaseController {

	@Autowired
	private SysUserService sysUserService;
	@Autowired
	private SysRoleService sysRoleService;
	@Autowired
	private SysUserRoleService sysUserRoleService;

	@ApiOperation("根据用户名查询用户信息接口")
	@ApiImplicitParam(name = "username", value = "用户名", required = true, dataType = "String", paramType = "path")
	@GetMapping("/{username}")
	public Object getUser(@PathVariable("username") String username) {
		return succeed(sysUserService.queryUserInfoByUsername(username));
	}

	@ApiOperation(value = "分页查询用户信息")
	@ApiImplicitParams({ @ApiImplicitParam(name = "userType", value = "用户类型", required = false, dataType = "String"),
			@ApiImplicitParam(name = "createStartDate", value = "开始注册时间", required = true, dataType = "String"),
			@ApiImplicitParam(name = "createEndDate", value = "结束注册时间", required = true, dataType = "String") })
	@GetMapping("/queryPage")
	public Object queryPage(@RequestBody SysUserQueryInfo queryInfo) {
		return succeed(sysUserService.queryPage(queryInfo));
	}

	@ApiOperation(value = "根据用户编号查询用户信息接口")
	@GetMapping("/query/{id}")
	public Object query(@ApiParam(value = "用户的昵称", required = true) @PathVariable("id") Integer id) {
		return succeed(sysUserService.get(id));
	}

	@ApiOperation(value = "新增用户")
	@PostMapping("/add")
	public Object add(@RequestBody SysUser sysUser) {
		Date date = new Date();
		sysUser.setCreateTime(date);
		sysUser.setUpdateTime(date);
		sysUser.setPassword(new BCryptPasswordEncoder().encode(sysUser.getPassword()));
		sysUserService.insert(sysUser);
		return succeed();
	}

	@ApiOperation(value = "删除用户")
	@DeleteMapping("/del/{id}")
	public Object del(@ApiParam(value = "用户编号", required = true) @PathVariable("id") Integer id) {
		return sysUserService.deleteById(id) ? succeed("删除成功") : failed("删除失败");
	}

	@ApiOperation(value = "修改用户")
	@PutMapping("/update")
	public Object update(@RequestBody SysUser sysUser) {
		Date date = new Date();
		sysUser.setUpdateTime(date);
		sysUser.setPassword(new BCryptPasswordEncoder().encode(sysUser.getPassword()));
		sysUserService.update(sysUser);
		return succeed();
	}

	@ApiOperation(value = "获取用户角色列表")
	@GetMapping("/getRole/{id}")
	public Object getRole(@ApiParam(value = "用户编号", required = true) @PathVariable("id") Integer id) {
		return succeed(sysRoleService.findRoleByUserId(id));
	}

	@ApiOperation(value = "用户角色新增")
	@ApiImplicitParams({ @ApiImplicitParam(name = "userId", value = "用户编号", required = true, dataType = "Integer"),
			@ApiImplicitParam(name = "roleIds", value = "角色id,逗号分隔", required = true, dataType = "String") })
	@PostMapping("/addRole")
	public Object getRole(@RequestBody HashMap<String,String> param) {
		String userId = param.get("userId");
		String roleIds = param.get("roleIds");
		if(StringUtils.isEmpty(param.get("userId")) || StringUtils.isEmpty(param.get("roleIds"))){
			return failed(SecurityConstants.PARAM_VERIFY_EXCEPTION);
		}
		sysUserRoleService.batchInsert(Integer.parseInt(userId),roleIds);
		return succeed();
	}

	@ApiOperation(value = "用户角色删除")
	@ApiImplicitParams({ @ApiImplicitParam(name = "userId", value = "用户编号", required = true, dataType = "Integer"),
			@ApiImplicitParam(name = "roleIds", value = "角色id,逗号分隔", required = true, dataType = "String") })
	@DeleteMapping("/delRole")
	public Object delRole(@RequestBody HashMap<String,String> param) {
		String userId = param.get("userId");
		String roleIds = param.get("roleIds");
		if(StringUtils.isEmpty(param.get("userId")) || StringUtils.isEmpty(param.get("roleIds"))){
			return failed(SecurityConstants.PARAM_VERIFY_EXCEPTION);
		}
		sysUserRoleService.batchDel(Integer.parseInt(userId),roleIds);
		return succeed();
	}
}
