package com.ym.mec.web.controller;

import com.ym.mec.common.controller.BaseController;
import com.ym.mec.web.dal.entity.MusicGroup;
import com.ym.mec.web.dal.page.MusicGroupQueryInfo;
import com.ym.mec.web.dal.utilEntity.SubSettingUtilEntity;
import com.ym.mec.web.service.MusicGroupService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import java.util.Date;
import java.util.List;

@RequestMapping("musicGroup")
@Api(tags = "乐团服务")
@RestController
public class MusicGroupController extends BaseController {

    @Autowired
    private MusicGroupService musicGroupService;

    @ApiOperation(value = "新增乐团")
    @PostMapping("/add")
    public Object add(@RequestBody MusicGroup musicGroup){
        Date date = new Date();
        musicGroup.setCreateTime(date);
        musicGroup.setUpdateTime(date);
        musicGroupService.insert(musicGroup);
        return succeed();
    }

    @ApiOperation(value = "删除乐团")
    @DeleteMapping("/del/{id}")
    public Object del(@ApiParam(value = "乐团编号", required = true) @PathVariable("id") String id){
        musicGroupService.delete(id);
        return succeed();
    }

    @ApiOperation(value = "修改乐团")
    @PutMapping("/update")
    public Object update(@RequestBody MusicGroup musicGroup){
        musicGroup.setUpdateTime(new Date());
        musicGroupService.update(musicGroup);
        return succeed();
    }

    @ApiOperation(value = "根据乐团编号查询乐团")
    @GetMapping("/get/{id}")
    public Object get(@ApiParam(value = "乐团编号", required = true) @PathVariable("id") String id){
        return succeed(musicGroupService.get(id));
    }

    @ApiOperation(value = "分页查询乐团列表")
    @PostMapping("/queryPage")
    public Object queryPage(@RequestBody MusicGroupQueryInfo queryInfo){
        return succeed(musicGroupService.queryPage(queryInfo));
    }

    @ApiOperation(value = "【建团申请-->】，新增、修改提交【声部设置】")
    @PostMapping("/subSet")
    public Object subSet(@RequestBody List<SubSettingUtilEntity> subSettingUtilEntitys){
        musicGroupService.subSetting(subSettingUtilEntitys);
        return succeed();
    }

    @ApiOperation(value = "建团申请，根据乐团编号获取已有声部设置")
    @PostMapping("/getSubSet")
    public Object getSubSetting(@RequestBody Integer musicGroupId){
        return succeed(musicGroupService.getSubSetting(musicGroupId));
    }
}
