package com.ym.mec.web.controller;

import com.ym.mec.common.controller.BaseController;
import com.ym.mec.web.dal.entity.Organization;
import com.ym.mec.web.dal.page.OrganizationQueryInfo;
import com.ym.mec.web.service.OrganizationService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Date;

@RequestMapping("organization")
@Api(tags = "组织机构服务")
@RestController
public class OrganizationController extends BaseController {

    @Autowired
    private OrganizationService organizationService;

    @ApiOperation(value = "分页查询组织机构树状列表")
    @PostMapping("/queryPage")
    public Object queryPage(@RequestBody OrganizationQueryInfo queryInfo){
        return organizationService.queryTreePage(queryInfo);
    }

    @ApiOperation(value = "新增机构")
    @PostMapping("/add")
    public Object add(@RequestBody Organization organization){
        Organization organizationByCode = organizationService.findByCode(organization.getCode());
        if(organizationByCode != null){
            return failed("组织机构代码已经存在,请更改");
        }
        Date date = new Date();
        organization.setCreateTime(date);
        organization.setUpdateTime(date);
        return succeed(organizationService.insert(organization));
    }

    @ApiOperation(value = "根据机构编号删除机构")
    @DeleteMapping("/del/{id}")
    public Object del(@ApiParam(value = "组织编号", required = true) @PathVariable("id") Integer id){
        return succeed(organizationService.delete(id));
    }

    @ApiOperation(value = "修改机构信息")
    @PutMapping("/update")
    public Object update(@RequestBody Organization organization){
        Organization organizationByCode = organizationService.findByCode(organization.getCode());
        if(organizationByCode != null && !organizationByCode.getId().equals(organization.getId())){
            return failed("组织机构代码已存在,请更改");
        }
        organization.setUpdateTime(new Date());
        return succeed(organizationService.update(organization));
    }

    @ApiOperation(value = "根据机构编号查询机构详情")
    @GetMapping("/get/{id}")
    public Object get(@ApiParam(value = "组织编号", required = true) @PathVariable("id") Integer id){
        return succeed(organizationService.get(id));
    }
}
