package com.ym.mec.web.controller;

import com.ym.mec.common.controller.BaseController;
import com.ym.mec.common.security.AuthUser;
import com.ym.mec.common.security.SecurityUtils;
import com.ym.mec.web.dal.entity.ClassGroup;
import com.ym.mec.web.dal.entity.Teacher;
import com.ym.mec.web.dal.page.TeacherQueryInfo;
import com.ym.mec.web.service.ClassGroupService;
import com.ym.mec.web.service.TeacherService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Date;
import java.util.List;

@RequestMapping("teacher")
@Api(tags = "教师服务")
@RestController
public class TeacherController extends BaseController {

    @Autowired
    private TeacherService teacherService;

    @Autowired
    private ClassGroupService classGroupService;

    @ApiOperation(value = "新增教师")
    @PostMapping("/add")
    public Object add(@RequestBody Teacher teacher) {
        Teacher teacherServiceByCertificateNum = teacherService.findByCertificateNum(teacher.getCertificateNum());
        if (teacherServiceByCertificateNum != null) {
            return failed("系统已存在该证件号的老师，请核查");
        }
        Integer organId = SecurityUtils.getUser().getOrganId();
        if(organId != null){
            teacher.setOrganId(organId);
        }
        Date date = new Date();
        teacher.setCreateTime(date);
        teacher.setUpdateTime(date);
        teacherService.insert(teacher);
        return succeed();
    }

    @ApiOperation(value = "删除教师")
    @DeleteMapping("/del/{id}")
    public Object del(@ApiParam(value = "教师编号", required = true) @PathVariable("id") Integer id) {
        teacherService.delete(id);
        return succeed();
    }

    @ApiOperation(value = "修改教师")
    @PutMapping("/update")
    public Object update(@RequestBody Teacher teacher) {
        Integer organId = SecurityUtils.getUser().getOrganId();
        if(organId != null){
            teacher.setOrganId(organId);
        }
        Teacher teacherServiceByCertificateNum = teacherService.findByCertificateNum(teacher.getCertificateNum());
        if (teacherServiceByCertificateNum != null && !teacherServiceByCertificateNum.getUserId().equals(teacher.getUserId())) {
            return failed("系统已存在该证件号的老师，请核查");
        }
        teacher.setUpdateTime(new Date());
        teacherService.update(teacher);
        return succeed();
    }

    @ApiOperation(value = "根据教师编号查询教师")
    @GetMapping("/get/{id}")
    public Object get(@ApiParam(value = "教师编号", required = true) @PathVariable("id") Integer id) {
        return succeed(teacherService.get(id));
    }

    @ApiOperation(value = "分页查询教师列表")
    @PostMapping("/queryPage")
    public Object queryPage(@RequestBody TeacherQueryInfo queryInfo) {
        return succeed(teacherService.queryPage(queryInfo));
    }

    @ApiOperation(value = "根据教师编号查询教师课程班列表")
    @GetMapping("/getClass/{id}")
    public Object getClassGroup(@ApiParam(value = "教师编号", required = true) @PathVariable("id") Integer id) {
        return succeed(classGroupService.findClassGroup4Teacher(id));
    }
}
